package net.serenitybdd.core.photography.integration

import com.google.common.collect.Lists
import net.serenitybdd.core.photography.Darkroom
import net.serenitybdd.core.photography.Photographer
import net.serenitybdd.core.photography.ScreenshotPhoto
import org.openqa.selenium.phantomjs.PhantomJSDriver
import spock.lang.Specification
import java.nio.file.Files
import java.nio.file.Path
import java.util.concurrent.Callable
import java.util.concurrent.ExecutorService
import java.util.concurrent.Executors
import java.util.concurrent.Future

class WhenAPhotographerTakesLotsOfScreenshots extends Specification {

    def "when a photographer takes a series of screenshots they all should be stored"() {
        given:
            def photographer = new Photographer();
        when:
            List<ScreenshotPhoto> photos = Lists.newArrayList();
            def driver
            try {
                driver = new PhantomJSDriver()
                Darkroom.isOpenForBusiness();
                for (photoCount in 0..10) {
                    photos.add(photographer.takesAScreenshot()
                        .with(driver)
                        .andSaveToDirectory(screenshotDirectory));
                }
                Darkroom.waitUntilClose();
            } finally {
                try {
                    if (driver) {
                        driver.close()
                        driver.quit()
                    }
                } catch (some) {
                }
            }
        then:
            photos.findAll {
                photo -> Files.exists(photo.pathToScreenshot)
            }.size() == 11

    }

    def "should handle multiple screenshots in parallel"() {
        given:
            def photographer = new Photographer();
            def List<Future<List<ScreenshotPhoto>>> processing = new ArrayList<>();
            def List<ScreenshotPhoto> screenshots = new ArrayList<>();
            def Integer threads = 10
            def Integer photos = 5

            def ExecutorService service = Executors.newFixedThreadPool(10);
        when:
            threads.times {
                processing << service.submit(new Callable<List<ScreenshotPhoto>>() {
                    @Override
                    List<ScreenshotPhoto> call() throws Exception {
                        def List<ScreenshotPhoto> photo = new ArrayList<>()
                        Darkroom.isOpenForBusiness();
                        def driver = null
                        try {
                            driver = new PhantomJSDriver()
                            driver.get(siteFromUrlAt("/static-site/index.html"))
                            photos.times {
                                photo.add(photographer.takesAScreenshot()
                                    .with(driver)
                                    .andSaveToDirectory(screenshotDirectory));
                            }
                        } catch (some) {
                            println some

                        } finally {
                            try {
                                if (driver) {
                                    driver.close()
                                    driver.quit()
                                }
                            } catch (some) {
                            }
                        }
                        Darkroom.waitUntilClose();
                        return photo
                    }
                })
            }
            processing.each { future ->
                screenshots.addAll(future.get())
            }
        then:
            processing.size() == threads
            screenshots.size() == threads * photos
            screenshots.findAll {
                photo -> Files.exists(photo.pathToScreenshot)
            }.size() == threads * photos
    }

    Path screenshotDirectory;

    long startTime;

    def setup() {
        screenshotDirectory = Files.createTempDirectory("screenshots")
        startTime = System.currentTimeMillis()

        Darkroom.isOpenForBusiness();
    }

    String siteFromUrlAt(String path) {
        File baseDir = new File(System.getProperty("user.dir"));
        File testSite = new File(baseDir, "src/test/resources" + path);
        return "file://" + testSite.getAbsolutePath();
    }

    def cleanup() {
        println "Test duration: " + (System.currentTimeMillis() - startTime) + " ms"
    }

}
